@@ -115,6 +115,7 @@ public:
     QQmlContextData(QQmlContext *);
     void emitDestruction();
     void clearContext();
+    void clearContextRecursively();
     void invalidate();
 
     inline bool isValid() const {
@@ -158,9 +159,9 @@ public:
     void initFromTypeCompilationUnit(const QQmlRefPointer<QV4::CompiledData::CompilationUnit> &unit, int subComponentIndex);
 
     // flag indicates whether the context owns the cache (after mutation) or not.
-    mutable QV4::IdentifierHash<int> propertyNameCache;
-    const QV4::IdentifierHash<int> &propertyNames() const;
-    QV4::IdentifierHash<int> &detachedPropertyNames();
+    mutable QV4::IdentifierHash propertyNameCache;
+    const QV4::IdentifierHash &propertyNames() const;
+    QV4::IdentifierHash &detachedPropertyNames();
 
     // Context object
     QObject *contextObject;
@@ -178,7 +179,7 @@ public:
     QQmlRefPointer<QQmlTypeNameCache> imports;
 
     // My children
-    QQmlContextData *childContexts = 0;
+    QQmlContextData *childContexts = nullptr;
 
     // My peers in parent's childContexts list
     QQmlContextData  *nextChild;
@@ -191,7 +192,7 @@ public:
     QQmlData *contextObjects;
 
     // Doubly-linked list of context guards (XXX merge with contextObjects)
-    QQmlGuardedContextData *contextGuards = 0;
+    QQmlGuardedContextData *contextGuards = nullptr;
 
     // id guards
     struct ContextGuard : public QQmlGuard<QObject>
@@ -261,9 +262,9 @@ private:
 
     inline void clear();
 
-    QQmlContextData *m_contextData = 0;
-    QQmlGuardedContextData  *m_next = 0;
-    QQmlGuardedContextData **m_prev = 0;
+    QQmlContextData *m_contextData = nullptr;
+    QQmlGuardedContextData  *m_next = nullptr;
+    QQmlGuardedContextData **m_prev = nullptr;
 };
 
 
@@ -287,14 +288,14 @@ void QQmlGuardedContextData::clear()
     if (m_prev) {
         *m_prev = m_next;
         if (m_next) m_next->m_prev = m_prev;
-        m_contextData = 0;
-        m_next = 0;
-        m_prev = 0;
+        m_contextData = nullptr;
+        m_next = nullptr;
+        m_prev = nullptr;
     }
 }
 
 QQmlContextDataRef::QQmlContextDataRef()
-    : m_contextData(0)
+    : m_contextData(nullptr)
 {
 }
 
@@ -338,7 +339,7 @@ void QQmlContextDataRef::clear()
 {
     if (m_contextData && !--m_contextData->refCount)
         m_contextData->destroy();
-    m_contextData = 0;
+    m_contextData = nullptr;
 }
 
 QQmlContextDataRef &
@@ -356,7 +357,7 @@ QQmlContextDataRef::operator=(const QQmlContextDataRef &other)
 }
 
 QQmlContextData::ContextGuard::ContextGuard()
-: context(0)
+: context(nullptr)
 {
 }
 
